@@ -7,28 +7,22 @@ from enum import Enum
 from itertools import cycle
 from pathlib import Path
 from textwrap import dedent
-from typing import Any, Dict, List, Optional, Union
+from typing import Annotated, Any, Optional, Union
 
 import fastar
 import rignore
 import typer
 from httpx import Client
-from pydantic import BaseModel, EmailStr, ValidationError
+from pydantic import BaseModel, EmailStr, TypeAdapter, ValidationError
 from rich.text import Text
 from rich_toolkit import RichToolkit
 from rich_toolkit.menu import Option
-from typing_extensions import Annotated
 
 from fastapi_cloud_cli.commands.login import login
 from fastapi_cloud_cli.utils.api import APIClient, BuildLogError, TooManyRetriesError
 from fastapi_cloud_cli.utils.apps import AppConfig, get_app_config, write_app_config
-from fastapi_cloud_cli.utils.auth import is_logged_in
+from fastapi_cloud_cli.utils.auth import Identity
 from fastapi_cloud_cli.utils.cli import get_rich_toolkit, handle_http_errors
-from fastapi_cloud_cli.utils.pydantic_compat import (
-    TypeAdapter,
-    model_dump,
-    model_validate,
-)
 
 logger = logging.getLogger(__name__)
 
@@ -57,6 +51,7 @@ def _should_exclude_entry(path: Path) -> bool:
         "__pycache__",
         ".mypy_cache",
         ".pytest_cache",
+        ".git",
         ".gitignore",
         ".fastapicloudignore",
     ]
@@ -102,14 +97,14 @@ class Team(BaseModel):
     name: str
 
 
-def _get_teams() -> List[Team]:
+def _get_teams() -> list[Team]:
     with APIClient() as client:
         response = client.get("/teams/")
         response.raise_for_status()
 
         data = response.json()["data"]
 
-    return [model_validate(Team, team) for team in data]
+    return [Team.model_validate(team) for team in data]
 
 
 class AppResponse(BaseModel):
@@ -126,7 +121,7 @@ def _create_app(team_id: str, app_name: str) -> AppResponse:
 
         response.raise_for_status()
 
-        return model_validate(AppResponse, response.json())
+        return AppResponse.model_validate(response.json())
 
 
 class DeploymentStatus(str, Enum):
@@ -179,12 +174,12 @@ def _create_deployment(app_id: str) -> CreateDeploymentResponse:
         response = client.post(f"/apps/{app_id}/deployments/")
         response.raise_for_status()
 
-        return model_validate(CreateDeploymentResponse, response.json())
+        return CreateDeploymentResponse.model_validate(response.json())
 
 
 class RequestUploadResponse(BaseModel):
     url: str
-    fields: Dict[str, str]
+    fields: dict[str, str]
 
 
 def _upload_deployment(deployment_id: str, archive_path: Path) -> None:
@@ -204,7 +199,7 @@ def _upload_deployment(deployment_id: str, archive_path: Path) -> None:
         response = fastapi_client.post(f"/deployments/{deployment_id}/upload")
         response.raise_for_status()
 
-        upload_data = model_validate(RequestUploadResponse, response.json())
+        upload_data = RequestUploadResponse.model_validate(response.json())
         logger.debug("Received upload URL: %s", upload_data.url)
 
         logger.debug("Starting file upload to S3")
@@ -239,17 +234,17 @@ def _get_app(app_slug: str) -> Optional[AppResponse]:
 
         data = response.json()
 
-    return model_validate(AppResponse, data)
+    return AppResponse.model_validate(data)
 
 
-def _get_apps(team_id: str) -> List[AppResponse]:
+def _get_apps(team_id: str) -> list[AppResponse]:
     with APIClient() as client:
         response = client.get("/apps/", params={"team_id": team_id})
         response.raise_for_status()
 
         data = response.json()["data"]
 
-    return [model_validate(AppResponse, app) for app in data]
+    return [AppResponse.model_validate(app) for app in data]
 
 
 WAITING_MESSAGES = [
@@ -300,6 +295,8 @@ def _configure_app(toolkit: RichToolkit, path_to_deploy: Path) -> AppConfig:
 
     toolkit.print_line()
 
+    selected_app: Optional[AppResponse] = None
+
     if not create_new_app:
         with toolkit.progress("Fetching apps...") as progress:
             with handle_http_errors(
@@ -316,18 +313,45 @@ def _configure_app(toolkit: RichToolkit, path_to_deploy: Path) -> AppConfig:
 
             raise typer.Exit(1)
 
-        app = toolkit.ask(
+        selected_app = toolkit.ask(
             "Select the app you want to deploy to:",
             options=[Option({"name": app.slug, "value": app}) for app in apps],
         )
-    else:
-        app_name = toolkit.input(
+
+    app_name = (
+        selected_app.slug
+        if selected_app
+        else toolkit.input(
             title="What's your app name?",
             default=_get_app_name(path_to_deploy),
         )
+    )
 
-        toolkit.print_line()
+    toolkit.print_line()
 
+    toolkit.print("Deployment configuration:", tag="summary")
+    toolkit.print_line()
+    toolkit.print(f"Team: [bold]{team.name}[/bold]")
+    toolkit.print(f"App name: [bold]{app_name}[/bold]")
+    toolkit.print_line()
+
+    choice = toolkit.ask(
+        "Does everything look right?",
+        tag="confirm",
+        options=[
+            Option({"name": "Yes, start the deployment!", "value": "deploy"}),
+            Option({"name": "No, let me start over", "value": "cancel"}),
+        ],
+    )
+    toolkit.print_line()
+
+    if choice == "cancel":
+        toolkit.print("Deployment cancelled.")
+        raise typer.Exit(0)
+
+    if selected_app:  # pragma: no cover
+        app = selected_app
+    else:
         with toolkit.progress(title="Creating app...") as progress:
             with handle_http_errors(progress):
                 app = _create_app(team.id, app_name)
@@ -363,9 +387,12 @@ def _wait_for_deployment(
 
     last_message_changed_at = time.monotonic()
 
-    with toolkit.progress(
-        next(messages), inline_logs=True, lines_to_show=20
-    ) as progress, APIClient() as client:
+    with (
+        toolkit.progress(
+            next(messages), inline_logs=True, lines_to_show=20
+        ) as progress,
+        APIClient() as client,
+    ):
         try:
             for log in client.stream_build_logs(deployment.id):
                 time_elapsed = time.monotonic() - started_at
@@ -376,13 +403,13 @@ def _wait_for_deployment(
                 if log.type == "complete":
                     progress.log("")
                     progress.log(
-                        f"🐔 Ready the chicken! Your app is ready at [link={deployment.url}]{deployment.url}[/link]"
+                        f"You can also check the app logs at [link={deployment.dashboard_url}]{deployment.dashboard_url}[/link]"
                     )
 
                     progress.log("")
 
                     progress.log(
-                        f"You can also check the app logs at [link={deployment.dashboard_url}]{deployment.dashboard_url}[/link]"
+                        f"🐔 Ready the chicken! Your app is ready at [link={deployment.url}]{deployment.url}[/link]"
                     )
 
                     break
@@ -432,7 +459,7 @@ def _send_waitlist_form(
     with toolkit.progress("Sending your request...") as progress:
         with APIClient() as client:
             with handle_http_errors(progress):
-                response = client.post("/users/waiting-list", json=model_dump(result))
+                response = client.post("/users/waiting-list", json=result.model_dump())
 
                 response.raise_for_status()
 
@@ -457,7 +484,7 @@ def _waitlist_form(toolkit: RichToolkit) -> None:
 
     toolkit.print_line()
 
-    result = model_validate(SignupToWaitingList, {"email": email})
+    result = SignupToWaitingList.model_validate({"email": email})
 
     if toolkit.confirm(
         "Do you want to get access faster by giving us more information?",
@@ -481,8 +508,7 @@ def _waitlist_form(toolkit: RichToolkit) -> None:
         result = form.run()  # type: ignore
 
         try:
-            result = model_validate(
-                SignupToWaitingList,
+            result = SignupToWaitingList.model_validate(
                 {
                     "email": email,
                     **result,  # type: ignore
@@ -531,15 +557,27 @@ def deploy(
     skip_wait: Annotated[
         bool, typer.Option("--no-wait", help="Skip waiting for deployment status")
     ] = False,
+    provided_app_id: Annotated[
+        Union[str, None],
+        typer.Option(
+            "--app-id",
+            help="Application ID to deploy to",
+            envvar="FASTAPI_CLOUD_APP_ID",
+        ),
+    ] = None,
 ) -> Any:
     """
     Deploy a [bold]FastAPI[/bold] app to FastAPI Cloud. 🚀
     """
     logger.debug("Deploy command started")
-    logger.debug("Deploy path: %s, skip_wait: %s", path, skip_wait)
+    logger.debug(
+        "Deploy path: %s, skip_wait: %s, app_id: %s", path, skip_wait, provided_app_id
+    )
+
+    identity = Identity()
 
     with get_rich_toolkit() as toolkit:
-        if not is_logged_in():
+        if not identity.is_logged_in():
             logger.debug("User not logged in, prompting for login or waitlist")
 
             toolkit.print_title("Welcome to FastAPI Cloud!", tag="FastAPI")
@@ -576,19 +614,43 @@ def deploy(
 
         app_config = get_app_config(path_to_deploy)
 
-        if not app_config:
+        if app_config and provided_app_id and app_config.app_id != provided_app_id:
+            toolkit.print(
+                f"[error]Error: Provided app ID ({provided_app_id}) does not match the local "
+                f"config ({app_config.app_id}).[/]"
+            )
+            toolkit.print_line()
+            toolkit.print(
+                "Run [bold]fastapi cloud unlink[/] to remove the local config, "
+                "or remove --app-id / unset FASTAPI_CLOUD_APP_ID to use the configured app.",
+                tag="tip",
+            )
+
+            raise typer.Exit(1) from None
+
+        if provided_app_id:
+            target_app_id = provided_app_id
+        elif app_config:
+            target_app_id = app_config.app_id
+        else:
             logger.debug("No app config found, configuring new app")
+
             app_config = _configure_app(toolkit, path_to_deploy=path_to_deploy)
             toolkit.print_line()
+
+            target_app_id = app_config.app_id
+
+        if provided_app_id:
+            toolkit.print(f"Deploying to app [blue]{target_app_id}[/blue]...")
         else:
-            logger.debug("Existing app config found, proceeding with deployment")
             toolkit.print("Deploying app...")
-            toolkit.print_line()
+
+        toolkit.print_line()
 
         with toolkit.progress("Checking app...", transient=True) as progress:
             with handle_http_errors(progress):
-                logger.debug("Checking app with ID: %s", app_config.app_id)
-                app = _get_app(app_config.app_id)
+                logger.debug("Checking app with ID: %s", target_app_id)
+                app = _get_app(target_app_id)
 
             if not app:
                 logger.debug("App not found in API")
@@ -598,10 +660,12 @@ def deploy(
 
         if not app:
             toolkit.print_line()
-            toolkit.print(
-                "If you deleted this app, you can run [bold]fastapi cloud unlink[/] to unlink the local configuration.",
-                tag="tip",
-            )
+
+            if not provided_app_id:
+                toolkit.print(
+                    "If you deleted this app, you can run [bold]fastapi cloud unlink[/] to unlink the local configuration.",
+                    tag="tip",
+                )
             raise typer.Exit(1)
 
         with tempfile.TemporaryDirectory() as temp_dir:
@@ -609,9 +673,10 @@ def deploy(
             archive_path = Path(temp_dir) / "archive.tar"
             archive(path or Path.cwd(), archive_path)
 
-            with toolkit.progress(
-                title="Creating deployment"
-            ) as progress, handle_http_errors(progress):
+            with (
+                toolkit.progress(title="Creating deployment") as progress,
+                handle_http_errors(progress),
+            ):
                 logger.debug("Creating deployment for app: %s", app.id)
                 deployment = _create_deployment(app.id)
 
