@@ -8,15 +8,8 @@ from pydantic import BaseModel
 
 from fastapi_cloud_cli.config import Settings
 from fastapi_cloud_cli.utils.api import APIClient
-from fastapi_cloud_cli.utils.auth import (
-    AuthConfig,
-    get_auth_token,
-    is_logged_in,
-    is_token_expired,
-    write_auth_config,
-)
+from fastapi_cloud_cli.utils.auth import AuthConfig, Identity, write_auth_config
 from fastapi_cloud_cli.utils.cli import get_rich_toolkit, handle_http_errors
-from fastapi_cloud_cli.utils.pydantic_compat import model_validate_json
 
 logger = logging.getLogger(__name__)
 
@@ -44,7 +37,7 @@ def _start_device_authorization(
 
     response.raise_for_status()
 
-    return model_validate_json(AuthorizationData, response.text)
+    return AuthorizationData.model_validate_json(response.text)
 
 
 def _fetch_access_token(client: httpx.Client, device_code: str, interval: int) -> str:
@@ -74,7 +67,7 @@ def _fetch_access_token(client: httpx.Client, device_code: str, interval: int) -
 
         time.sleep(interval)
 
-    response_data = model_validate_json(TokenResponse, response.text)
+    response_data = TokenResponse.model_validate_json(response.text)
 
     return response_data.access_token
 
@@ -83,17 +76,18 @@ def login() -> Any:
     """
     Login to FastAPI Cloud. 🚀
     """
-    token = get_auth_token()
-    if token is not None and is_token_expired(token):
+    identity = Identity()
+
+    if identity.is_expired():
         with get_rich_toolkit(minimal=True) as toolkit:
             toolkit.print("Your session has expired. Logging in again...")
             toolkit.print_line()
 
-    if is_logged_in():
+    if identity.is_logged_in():
         with get_rich_toolkit(minimal=True) as toolkit:
             toolkit.print("You are already logged in.")
             toolkit.print(
-                "Run [bold]fastapi logout[/bold] first if you want to switch accounts."
+                "Run [bold]fastapi cloud logout[/bold] first if you want to switch accounts."
             )
         return
 
