@@ -1,14 +1,16 @@
 import sentry_sdk
 from sentry_sdk.integrations.typer import TyperIntegration
 
-from .auth import is_logged_in
+from fastapi_cloud_cli.utils.auth import Identity
 
 SENTRY_DSN = "https://230250605ea4b58a0b69c768e9ec1168@o4506985151856640.ingest.us.sentry.io/4508449198899200"
 
 
 def init_sentry() -> None:
     """Initialize Sentry error tracking only if user is logged in."""
-    if not is_logged_in():
+    identity = Identity()
+
+    if not identity.is_logged_in():
         return
 
     sentry_sdk.init(
