@@ -2,7 +2,7 @@ import random
 import string
 from datetime import timedelta
 from pathlib import Path
-from typing import Dict, Optional
+from typing import Optional
 from unittest.mock import patch
 
 import httpx
@@ -25,7 +25,7 @@ settings = Settings.get()
 assets_path = Path(__file__).parent / "assets"
 
 
-def _get_random_team() -> Dict[str, str]:
+def _get_random_team() -> dict[str, str]:
     name = "".join(random.choices(string.ascii_lowercase, k=10))
     slug = "".join(random.choices(string.ascii_lowercase, k=10))
     id = "".join(random.choices(string.digits, k=10))
@@ -35,7 +35,7 @@ def _get_random_team() -> Dict[str, str]:
 
 def _get_random_app(
     *, slug: Optional[str] = None, team_id: Optional[str] = None
-) -> Dict[str, str]:
+) -> dict[str, str]:
     name = "".join(random.choices(string.ascii_lowercase, k=10))
     slug = slug or "".join(random.choices(string.ascii_lowercase, k=10))
     id = "".join(random.choices(string.digits, k=10))
@@ -48,7 +48,7 @@ def _get_random_deployment(
     *,
     app_id: Optional[str] = None,
     status: str = "waiting_upload",
-) -> Dict[str, str]:
+) -> dict[str, str]:
     id = "".join(random.choices(string.digits, k=10))
     slug = "".join(random.choices(string.ascii_lowercase, k=10))
     app_id = app_id or "".join(random.choices(string.digits, k=10))
@@ -91,11 +91,11 @@ def test_chooses_login_option_when_not_logged_in(
         },
     ).mock(return_value=Response(200, json={"access_token": "test_token_1234"}))
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar, patch(
-        "fastapi_cloud_cli.commands.login.typer.launch"
-    ) as mock_launch:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+        patch("fastapi_cloud_cli.commands.login.typer.launch") as mock_launch,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -136,9 +136,10 @@ def test_chooses_waitlist_option_when_not_logged_in(
         },
     ).mock(return_value=Response(200))
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -200,9 +201,10 @@ def test_shows_waitlist_form_when_not_logged_in_longer_flow(
         },
     ).mock(return_value=Response(200))
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -220,9 +222,10 @@ def test_shows_error_when_trying_to_get_teams(
 
     respx_mock.get("/teams/").mock(return_value=Response(500))
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -240,9 +243,10 @@ def test_handles_invalid_auth(
 
     respx_mock.get("/teams/").mock(return_value=Response(401))
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -268,9 +272,10 @@ def test_shows_teams(
         )
     )
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -294,9 +299,10 @@ def test_asks_for_app_name_after_team(
         )
     )
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -310,7 +316,7 @@ def test_asks_for_app_name_after_team(
 def test_creates_app_on_backend(
     logged_in_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
 ) -> None:
-    steps = [Keys.ENTER, Keys.ENTER, *"demo", Keys.ENTER]
+    steps = [Keys.ENTER, Keys.ENTER, *"demo", Keys.ENTER, Keys.ENTER]
 
     team = _get_random_team()
 
@@ -325,9 +331,10 @@ def test_creates_app_on_backend(
         return_value=Response(201, json=_get_random_app(team_id=team["id"]))
     )
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -337,6 +344,40 @@ def test_creates_app_on_backend(
         assert "App created successfully" in result.output
 
 
+@pytest.mark.respx(base_url=settings.base_api_url)
+def test_cancels_deployment_when_user_selects_no(
+    logged_in_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
+) -> None:
+    steps = [
+        Keys.ENTER,
+        Keys.ENTER,
+        *"demo",
+        Keys.ENTER,
+        Keys.DOWN_ARROW,
+        Keys.ENTER,
+    ]
+
+    team = _get_random_team()
+
+    respx_mock.get("/teams/").mock(
+        return_value=Response(
+            200,
+            json={"data": [team]},
+        )
+    )
+
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
+        mock_getchar.side_effect = steps
+
+        result = runner.invoke(app, ["deploy"])
+
+        assert result.exit_code == 0
+        assert "Deployment cancelled." in result.output
+
+
 @pytest.mark.respx(base_url=settings.base_api_url)
 def test_uses_existing_app(
     logged_in_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
@@ -353,9 +394,10 @@ def test_uses_existing_app(
         return_value=Response(200, json={"data": [app_data]})
     )
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -375,6 +417,7 @@ def test_exits_successfully_when_deployment_is_done(
         Keys.ENTER,
         *"demo",
         Keys.ENTER,
+        Keys.ENTER,
     ]
 
     team_data = _get_random_team()
@@ -426,9 +469,10 @@ def test_exits_successfully_when_deployment_is_done(
         )
     )
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -624,6 +668,7 @@ def _deploy_without_waiting(respx_mock: respx.MockRouter, tmp_path: Path) -> Res
         Keys.ENTER,
         *"demo",
         Keys.ENTER,
+        Keys.ENTER,
     ]
 
     team_data = _get_random_team()
@@ -668,9 +713,10 @@ def _deploy_without_waiting(respx_mock: respx.MockRouter, tmp_path: Path) -> Res
         return_value=Response(200)
     )
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         return runner.invoke(app, ["deploy", "--no-wait"])
@@ -723,9 +769,11 @@ def test_shows_error_for_invalid_waitlist_form_data(
         Keys.CTRL_C,  # Interrupt to avoid infinite loop
     ]
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar, patch("rich_toolkit.form.Form.run") as mock_form_run:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+        patch("rich_toolkit.form.Form.run") as mock_form_run,
+    ):
         mock_getchar.side_effect = steps
         # Simulate form returning data with invalid email field to trigger ValidationError
         mock_form_run.return_value = {
@@ -758,9 +806,10 @@ def test_shows_no_apps_found_message_when_team_has_no_apps(
         return_value=Response(200, json={"data": []})
     )
 
-    with changing_dir(tmp_path), patch(
-        "rich_toolkit.container.getchar"
-    ) as mock_getchar:
+    with (
+        changing_dir(tmp_path),
+        patch("rich_toolkit.container.getchar") as mock_getchar,
+    ):
         mock_getchar.side_effect = steps
 
         result = runner.invoke(app, ["deploy"])
@@ -806,9 +855,12 @@ def test_shows_error_message_on_build_exception(
         return_value=Response(200)
     )
 
-    with changing_dir(tmp_path), patch(
-        "fastapi_cloud_cli.utils.api.APIClient.stream_build_logs",
-        side_effect=error,
+    with (
+        changing_dir(tmp_path),
+        patch(
+            "fastapi_cloud_cli.utils.api.APIClient.stream_build_logs",
+            side_effect=error,
+        ),
     ):
         result = runner.invoke(app, ["deploy"])
 
@@ -1017,9 +1069,12 @@ def test_calls_upload_cancelled_when_user_interrupts(
         f"/deployments/{deployment_data['id']}/upload-cancelled"
     ).mock(return_value=Response(200))
 
-    with changing_dir(tmp_path), patch(
-        "fastapi_cloud_cli.commands.deploy._upload_deployment",
-        side_effect=KeyboardInterrupt(),
+    with (
+        changing_dir(tmp_path),
+        patch(
+            "fastapi_cloud_cli.commands.deploy._upload_deployment",
+            side_effect=KeyboardInterrupt(),
+        ),
     ):
         runner.invoke(app, ["deploy"])
 
@@ -1049,11 +1104,300 @@ def test_cancel_upload_swallows_exceptions(
         f"/deployments/{deployment_data['id']}/upload-cancelled"
     ).mock(return_value=Response(500))
 
-    with changing_dir(tmp_path), patch(
-        "fastapi_cloud_cli.commands.deploy._upload_deployment",
-        side_effect=KeyboardInterrupt(),
+    with (
+        changing_dir(tmp_path),
+        patch(
+            "fastapi_cloud_cli.commands.deploy._upload_deployment",
+            side_effect=KeyboardInterrupt(),
+        ),
     ):
         result = runner.invoke(app, ["deploy"])
 
         assert upload_cancelled_route.called
         assert "HTTPStatusError" not in result.output
+
+
+@pytest.mark.respx(base_url=settings.base_api_url)
+def test_deploy_successfully_with_token(
+    logged_out_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
+) -> None:
+    app_data = _get_random_app()
+    team_data = _get_random_team()
+    app_id = app_data["id"]
+    team_id = team_data["id"]
+    deployment_data = _get_random_deployment(app_id=app_id)
+
+    config_path = tmp_path / ".fastapicloud" / "cloud.json"
+
+    config_path.parent.mkdir(parents=True, exist_ok=True)
+    config_path.write_text(f'{{"app_id": "{app_id}", "team_id": "{team_id}"}}')
+
+    respx_mock.get(f"/apps/{app_id}", headers={"Authorization": "Bearer hello"}).mock(
+        return_value=Response(200, json=app_data)
+    )
+
+    respx_mock.post(
+        f"/apps/{app_id}/deployments/", headers={"Authorization": "Bearer hello"}
+    ).mock(return_value=Response(201, json=deployment_data))
+
+    respx_mock.post(
+        f"/deployments/{deployment_data['id']}/upload",
+        headers={"Authorization": "Bearer hello"},
+    ).mock(
+        return_value=Response(
+            200,
+            json={"url": "http://test.com", "fields": {"key": "value"}},
+        )
+    )
+
+    respx_mock.post("http://test.com", data={"key": "value"}).mock(
+        return_value=Response(200)
+    )
+
+    respx_mock.get(
+        f"/deployments/{deployment_data['id']}/build-logs",
+        headers={"Authorization": "Bearer hello"},
+    ).mock(
+        return_value=Response(
+            200,
+            content=build_logs_response(
+                {"type": "message", "message": "Building...", "id": "1"},
+                {"type": "message", "message": "All good!", "id": "2"},
+                {"type": "complete"},
+            ),
+        )
+    )
+
+    respx_mock.post(
+        f"/deployments/{deployment_data['id']}/upload-complete",
+        headers={"Authorization": "Bearer hello"},
+    ).mock(return_value=Response(200))
+
+    with changing_dir(tmp_path):
+        result = runner.invoke(app, ["deploy"], env={"FASTAPI_CLOUD_TOKEN": "hello"})
+
+        assert result.exit_code == 0
+
+        # check that logs are shown
+        assert "All good!" in result.output
+
+        # check that the dashboard URL is shown
+        assert "You can also check the app logs at" in result.output
+        assert deployment_data["dashboard_url"] in result.output
+
+        # check that the app URL is shown
+        assert deployment_data["url"] in result.output
+
+
+@pytest.mark.respx(base_url=settings.base_api_url)
+def test_deploy_with_token_fails(
+    logged_out_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
+) -> None:
+    app_data = _get_random_app()
+    team_data = _get_random_team()
+    app_id = app_data["id"]
+    team_id = team_data["id"]
+
+    config_path = tmp_path / ".fastapicloud" / "cloud.json"
+
+    config_path.parent.mkdir(parents=True, exist_ok=True)
+    config_path.write_text(f'{{"app_id": "{app_id}", "team_id": "{team_id}"}}')
+
+    respx_mock.get(f"/apps/{app_id}", headers={"Authorization": "Bearer hello"}).mock(
+        return_value=Response(401, json=app_data)
+    )
+
+    with changing_dir(tmp_path):
+        result = runner.invoke(app, ["deploy"], env={"FASTAPI_CLOUD_TOKEN": "hello"})
+
+        assert result.exit_code == 1
+
+        assert (
+            "The specified token is not valid. Make sure to use a valid token."
+            in result.output
+        )
+
+
+@pytest.mark.respx(base_url=settings.base_api_url)
+def test_deploy_with_app_id_arg(
+    logged_in_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
+) -> None:
+    app_data = _get_random_app()
+    app_id = app_data["id"]
+    deployment_data = _get_random_deployment(app_id=app_id)
+
+    respx_mock.get(f"/apps/{app_id}").mock(return_value=Response(200, json=app_data))
+
+    respx_mock.post(f"/apps/{app_id}/deployments/").mock(
+        return_value=Response(201, json=deployment_data)
+    )
+
+    respx_mock.post(f"/deployments/{deployment_data['id']}/upload").mock(
+        return_value=Response(
+            200,
+            json={"url": "http://test.com", "fields": {"key": "value"}},
+        )
+    )
+
+    respx_mock.post("http://test.com", data={"key": "value"}).mock(
+        return_value=Response(200)
+    )
+
+    respx_mock.post(f"/deployments/{deployment_data['id']}/upload-complete").mock(
+        return_value=Response(200)
+    )
+
+    respx_mock.get(f"/deployments/{deployment_data['id']}/build-logs").mock(
+        return_value=Response(
+            200,
+            content=build_logs_response(
+                {"type": "message", "message": "Building...", "id": "1"},
+                {"type": "complete"},
+            ),
+        )
+    )
+
+    with changing_dir(tmp_path):
+        result = runner.invoke(app, ["deploy", "--app-id", app_id])
+
+        assert result.exit_code == 0
+        assert f"Deploying to app {app_id}" in result.output
+
+
+@pytest.mark.respx(base_url=settings.base_api_url)
+def test_deploy_with_app_id_from_env_var(
+    logged_in_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
+) -> None:
+    app_data = _get_random_app()
+    app_id = app_data["id"]
+    deployment_data = _get_random_deployment(app_id=app_id)
+
+    respx_mock.get(f"/apps/{app_id}").mock(return_value=Response(200, json=app_data))
+
+    respx_mock.post(f"/apps/{app_id}/deployments/").mock(
+        return_value=Response(201, json=deployment_data)
+    )
+
+    respx_mock.post(f"/deployments/{deployment_data['id']}/upload").mock(
+        return_value=Response(
+            200,
+            json={"url": "http://test.com", "fields": {"key": "value"}},
+        )
+    )
+
+    respx_mock.post("http://test.com", data={"key": "value"}).mock(
+        return_value=Response(200)
+    )
+
+    respx_mock.post(f"/deployments/{deployment_data['id']}/upload-complete").mock(
+        return_value=Response(200)
+    )
+
+    respx_mock.get(f"/deployments/{deployment_data['id']}/build-logs").mock(
+        return_value=Response(
+            200,
+            content=build_logs_response(
+                {"type": "message", "message": "Building...", "id": "1"},
+                {"type": "complete"},
+            ),
+        )
+    )
+
+    with changing_dir(tmp_path):
+        result = runner.invoke(app, ["deploy"], env={"FASTAPI_CLOUD_APP_ID": app_id})
+
+        assert result.exit_code == 0
+        assert f"Deploying to app {app_id}" in result.output
+
+
+@pytest.mark.respx(base_url=settings.base_api_url)
+def test_deploy_with_app_id_matching_local_config(
+    logged_in_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
+) -> None:
+    app_data = _get_random_app()
+    app_id = app_data["id"]
+    team_id = "some-team-id"
+    deployment_data = _get_random_deployment(app_id=app_id)
+
+    config_path = tmp_path / ".fastapicloud" / "cloud.json"
+    config_path.parent.mkdir(parents=True, exist_ok=True)
+    config_path.write_text(f'{{"app_id": "{app_id}", "team_id": "{team_id}"}}')
+
+    respx_mock.get(f"/apps/{app_id}").mock(return_value=Response(200, json=app_data))
+
+    respx_mock.post(f"/apps/{app_id}/deployments/").mock(
+        return_value=Response(201, json=deployment_data)
+    )
+
+    respx_mock.post(f"/deployments/{deployment_data['id']}/upload").mock(
+        return_value=Response(
+            200,
+            json={"url": "http://test.com", "fields": {"key": "value"}},
+        )
+    )
+
+    respx_mock.post("http://test.com", data={"key": "value"}).mock(
+        return_value=Response(200)
+    )
+
+    respx_mock.post(f"/deployments/{deployment_data['id']}/upload-complete").mock(
+        return_value=Response(200)
+    )
+
+    respx_mock.get(f"/deployments/{deployment_data['id']}/build-logs").mock(
+        return_value=Response(
+            200,
+            content=build_logs_response(
+                {"type": "message", "message": "Building...", "id": "1"},
+                {"type": "complete"},
+            ),
+        )
+    )
+
+    with changing_dir(tmp_path):
+        result = runner.invoke(app, ["deploy", "--app-id", app_id])
+
+        assert result.exit_code == 0
+        # Should NOT show mismatch warning
+        assert "does not match" not in result.output
+        assert f"Deploying to app {app_id}" in result.output
+
+
+@pytest.mark.respx(base_url=settings.base_api_url)
+def test_deploy_with_app_id_mismatch_fails(
+    logged_in_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
+) -> None:
+    local_app_data = _get_random_app()
+    local_app_id = local_app_data["id"]
+    team_id = "some-team-id"
+
+    config_path = tmp_path / ".fastapicloud" / "cloud.json"
+    config_path.parent.mkdir(parents=True, exist_ok=True)
+    config_path.write_text(f'{{"app_id": "{local_app_id}", "team_id": "{team_id}"}}')
+
+    cli_app_id = "different-app-id"
+
+    with changing_dir(tmp_path):
+        result = runner.invoke(app, ["deploy", "--app-id", cli_app_id])
+
+        assert result.exit_code == 1
+        assert "does not match" in result.output
+        assert "fastapi cloud unlink" in result.output
+        assert "FASTAPI_CLOUD_APP_ID" in result.output
+
+
+@pytest.mark.respx(base_url=settings.base_api_url)
+def test_deploy_with_app_id_arg_app_not_found(
+    logged_in_cli: None, tmp_path: Path, respx_mock: respx.MockRouter
+) -> None:
+    app_id = "nonexistent-app-id"
+
+    respx_mock.get(f"/apps/{app_id}").mock(return_value=Response(404))
+
+    with changing_dir(tmp_path):
+        result = runner.invoke(app, ["deploy", "--app-id", app_id])
+
+        assert result.exit_code == 1
+        assert "App not found" in result.output
+        # Should NOT show unlink tip when using --app-id
+        assert "unlink" not in result.output
